#!/usr/bin/env python3

import requests
import time
import codecs
from lxml import html

class GihyoCrawler():
    def __init__(self):
        self.BASE_URL = "http://gihyo.jp"
        self.CONTENT_LIST_URL = "http://gihyo.jp/contentslist"
        self.TITLE_XPATH = """//*[@id="latestArticle"]/dl/dd/h3/a"""
        self.NEXT_URL_XPATH = """//*[@id="latestArticle"]/div[4]/p[2]/a"""
        self.OUTPUT_FILE = "output.tsv"
        self.PAGE_NUM = 2
        self.SLEEP_TIME = 1

    def run(self):
        url = self.CONTENT_LIST_URL
        with codecs.open(self.OUTPUT_FILE, "w", "utf-8") as f:
            for i in range(self.PAGE_NUM):
                dom = self.read_html(url, self.BASE_URL)
                for title, url in self.scrape_content_page(dom):
                    f.write(title + "\t" + url + "\n")
                url = self.get_next_url(dom)
                if url is None:
                    break
                time.sleep(self.SLEEP_TIME)

    def read_html(self, url, base_url):
        res = requests.get(url)
        dom = html.fromstring(res.text)
        return html.make_links_absolute(dom, base_url)

    def scrape_content_page(self, dom):
        titles = self.get_title(dom)
        urls = self.get_article_url(dom)
        for title, url in zip(titles, urls):
            yield (title, url)

    def get_title(self, dom):
        for i in dom.xpath(self.TITLE_XPATH):
            yield i.text_content().strip()

    def get_article_url(self, dom):
        for i in dom.xpath(self.TITLE_XPATH):
            yield i.get("href")

    def get_next_url(self, dom):
        try:
            return dom.xpath(self.NEXT_URL_XPATH)[0].get("href")
        except:
            return None

if __name__ == "__main__":
    crawler = GihyoCrawler()
    crawler.run()
